<?php
/**
 * @file
 * Contains admin forms for Search Engine Referers.
 */

/**
 * System Settings Form for Module.
 */
function search_engine_referers_settings() {
  $form = array();
  $form['search_engine_referers_db_limit'] = array(
    '#type' => 'select',
    '#title' => t('Maximum number of entries per search engine to display'),
    '#options' => array(
      5 => 5,
      10 => 10,
      15 => 15,
      20 => 20,
      30 => 30,
      50 => 50,
    ),
    '#default_value' => variable_get('search_engine_referers_db_limit', 10),
  );
  return system_settings_form($form);
}

/**
 * Callback function for overview page.
 */
function search_engine_referers_overview() {
  // Check if access logging is enabled.
  if (!variable_get('statistics_enable_access_log', 0)) {
    drupal_set_message(t('The access log is not enabled. The listing of search engine referers will be empty or outdated. <a href="!accesslog">Enable and configure the access log</a>.', array('!accesslog' => url('admin/reports/settings'))), 'warning');
  }

  $output = '';

  // Search engine patterns and properties.
  $engines = array(
    'Google' => array(
      'host_pattern_sql' => 'http://www.google.%',
      'host_pattern_preg' => '!google(\\.[a-z]+)+$!',
      'path_pattern' => '!^/(search|url)!',
      'query_variable' => 'q',
    ),
    'Yahoo!' => array(
      'host_pattern_sql' => 'http://%search.yahoo.com/%',
      'host_pattern_preg' => '!([a-z]+\\.)*search\\.yahoo\\.com!',
      'path_pattern' => '!^/search!',
      'query_variable' => 'p',
    ),
    'Live search' => array(
      'host_pattern_sql' => 'http://search.%.com/results.aspx%',
      'host_pattern_preg' => '!search\\.(live|msn)\\.com!',
      'path_pattern' => '!^/results.aspx!',
      'query_variable' => 'q',
    ),
    'Bing' => array(
      'host_pattern_sql' => 'http://%.bing.com/search%',
      'host_pattern_preg' => '!bing\\.com!',
      'path_pattern' => '!^/search!',
      'query_variable' => 'q',
    ),
    'Yandex' => array(
      'host_pattern_sql' => 'http://%yandex.ru/yandsearch%',
      'host_pattern_preg' => '!yandex\\.ru!',
      'path_pattern' => '!^/yandsearch!',
      'query_variable' => 'text',
    ),
    'Rambler' => array(
      'host_pattern_sql' => 'http://www.rambler.ru/srch%',
      'host_pattern_preg' => '!rambler\\.ru!',
      'path_pattern' => '!^/srch!',
      'query_variable' => 'words',
    ),
  );
  $num = intval(variable_get('search_engine_referers_db_limit', 10));

  // Build the table.
  $header = array(t('Date'), t('Page'), t('Search engine'), t('Query'));
  $rows = array();

  // Iterate over the engines.
  foreach ($engines as $engine_name => $engine) {
    $result = db_query_range("SELECT aid, timestamp, url, path FROM {accesslog} WHERE LOWER(url) LIKE LOWER('%s') ORDER BY timestamp DESC", $engine['host_pattern_sql'], 0, array($num))->fetchAll();
    foreach ($result as $r) {
      $url_data = parse_url($r->url);
      if (preg_match($engine['host_pattern_preg'], $url_data['host']) && preg_match($engine['path_pattern'], $url_data['path'])) {
        parse_str($url_data['query'], $query_data);
        // Create link to referer page.
        $url = l($query_data[$engine['query_variable']], $r->url);
        // Fix empty links, which can happen if query_variable isn't found or in
        // case of non UTF encoding of the query.
        if (substr($url, -5) == '></a>') {
          $url = l(t('[n/a]'), $r->url);
        }
        // Page.
        $title = drupal_get_path_alias($r->path);
        $title = truncate_utf8($title, 30, FALSE, TRUE);
        // Build row.
        $rows[$r->aid] = array(
          format_date($r->timestamp, 'small'),
          l($title, $r->path),
          $url_data['host'],
          $url,
        );
      }
    }
  }
  // Sort rows by date (access id actually).
  krsort($rows);

  if (empty($rows)) {
    $rows[] = array(array('data' => t('No search engine referers found.'), 'colspan' => 4));
  }

  // Theme the table.
  $output .= theme('table', $header, $rows);

  // Add settings form.
  $form = drupal_get_form('search_engine_referers_settings');
  $output .= render($form);

  return $output;
}
